"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var runtime_exports = {};
__export(runtime_exports, {
  handleRequest: () => handleRequest,
  isRedirectResponse: () => isRedirectResponse,
  isResponse: () => isResponse
});
module.exports = __toCommonJS(runtime_exports);
var import_node = require("@remix-run/node");
var import_remix_router = require("@alola-react/runtime-utils/remix-router");
var import_browser = require("@alola-react/runtime-utils/browser");
var import_lodash = require("@alola-react/utils/lodash");
var import_node2 = require("@alola-react/runtime-utils/node");
var import_time = require("@alola-react/runtime-utils/time");
var import_constants = require("@alola-react/utils/universal/constants");
var import_constants2 = require("../common/constants");
var import_response = require("./response");
(0, import_node.installGlobals)();
const redirectStatusCodes = /* @__PURE__ */ new Set([
  301,
  302,
  303,
  307,
  308
]);
function isRedirectResponse(status) {
  return redirectStatusCodes.has(status);
}
function isResponse(value) {
  return value != null && typeof value.status === "number" && typeof value.statusText === "string" && typeof value.headers === "object" && typeof value.body !== "undefined";
}
function convertAlolaRedirectResponse(headers, basename) {
  const newHeaders = new Headers(headers);
  let redirectUrl = headers.get("Location");
  if (basename !== "/") {
    redirectUrl = redirectUrl.replace(basename, "");
  }
  newHeaders.set("X-Alolajs-Redirect", redirectUrl);
  newHeaders.delete("Location");
  return new import_node.Response(null, {
    status: 204,
    headers: newHeaders
  });
}
const createLoaderHeaders = (requestHeaders) => {
  const headers = new Headers();
  for (const [key, values] of Object.entries(requestHeaders)) {
    if (values) {
      if (Array.isArray(values)) {
        for (const value of values) {
          headers.append(key, value);
        }
      } else {
        headers.set(key, values);
      }
    }
  }
  return headers;
};
const createRequest = (context) => {
  const origin = `${context.protocol}://${context.host}`;
  const url = new URL(context.url, origin);
  const controller = new AbortController();
  const init = {
    method: context.method,
    headers: createLoaderHeaders(context.headers),
    signal: controller.signal
  };
  if (![
    "GET",
    "HEAD"
  ].includes(context.method.toUpperCase())) {
    init.body = context.req;
  }
  const request = new Request(url.href, init);
  return request;
};
const sendLoaderResponse = async (res, nodeResponse) => {
  res.statusMessage = nodeResponse.statusText;
  res.statusCode = nodeResponse.status;
  for (const [key, value] of nodeResponse.headers.entries()) {
    res.setHeader(key, value);
  }
  if (nodeResponse.body) {
    await (0, import_node.writeReadableStreamToWritable)(nodeResponse.body, res);
  } else {
    res.end();
  }
};
const handleRequest = async ({ context, serverRoutes, routes: routesConfig }) => {
  const { query } = context;
  const routeId = query[import_constants2.LOADER_ID_PARAM];
  const entry = (0, import_node2.matchEntry)(context.path, serverRoutes);
  if (!routeId || !entry) {
    return;
  }
  const basename = entry.urlPath;
  const end = (0, import_time.time)();
  const { res, logger, reporter } = context;
  const routes = (0, import_browser.transformNestedRoutes)(routesConfig, reporter);
  const { queryRoute } = (0, import_remix_router.createStaticHandler)(routes, {
    basename
  });
  const request = createRequest(context);
  const requestContext = (0, import_node2.createRequestContext)();
  requestContext.set(import_node2.reporterCtx, reporter);
  let response;
  try {
    response = await queryRoute(request, {
      routeId,
      requestContext
    });
    if (isResponse(response) && isRedirectResponse(response.status)) {
      response = convertAlolaRedirectResponse(response.headers, basename);
    } else if ((0, import_lodash.isPlainObject)(response) && import_remix_router.UNSAFE_DEFERRED_SYMBOL in response) {
      const deferredData = response[import_remix_router.UNSAFE_DEFERRED_SYMBOL];
      const body = (0, import_response.createDeferredReadableStream)(deferredData, request.signal);
      const init = deferredData.init || {};
      if (init.status && isRedirectResponse(init.status)) {
        if (!init.headers) {
          throw new Error("redirect response includes no headers");
        }
        response = convertAlolaRedirectResponse(new Headers(init.headers), basename);
      } else {
        const headers = new Headers(init.headers);
        headers.set("Content-Type", `${import_constants2.CONTENT_TYPE_DEFERRED}; charset=UTF-8`);
        init.headers = headers;
        response = new import_node.Response(body, init);
      }
    } else {
      response = isResponse(response) ? response : new import_node.Response(JSON.stringify(response), {
        headers: {
          "Content-Type": "application/json; charset=utf-8"
        }
      });
    }
  } catch (error) {
    const message = (0, import_remix_router.isRouteErrorResponse)(error) ? error.data : String(error);
    if (error instanceof Error) {
      logger === null || logger === void 0 ? void 0 : logger.error(`Error: ${error.name} ${error.message} ${error.stack}`);
    } else {
      logger === null || logger === void 0 ? void 0 : logger.error(message);
    }
    response = new import_node.Response(message, {
      status: 500,
      headers: {
        "Content-Type": "text/plain"
      }
    });
  }
  const cost = end();
  reporter.reportTiming(`${import_constants.LOADER_REPORTER_NAME}-navigation`, cost);
  await sendLoaderResponse(res, response);
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  handleRequest,
  isRedirectResponse,
  isResponse
});
