import { _ as _async_to_generator } from "@swc/helpers/_/_async_to_generator";
import { _ as _instanceof } from "@swc/helpers/_/_instanceof";
import { _ as _sliced_to_array } from "@swc/helpers/_/_sliced_to_array";
import { _ as _ts_generator } from "@swc/helpers/_/_ts_generator";
import { installGlobals, writeReadableStreamToWritable, Response as NodeResponse } from "@remix-run/node";
import { createStaticHandler, UNSAFE_DEFERRED_SYMBOL as DEFERRED_SYMBOL, isRouteErrorResponse } from "@alola-react/runtime-utils/remix-router";
import { transformNestedRoutes } from "@alola-react/runtime-utils/browser";
import { isPlainObject } from "@alola-react/utils/lodash";
import { matchEntry, createRequestContext, reporterCtx } from "@alola-react/runtime-utils/node";
import { time } from "@alola-react/runtime-utils/time";
import { LOADER_REPORTER_NAME } from "@alola-react/utils/universal/constants";
import { CONTENT_TYPE_DEFERRED, LOADER_ID_PARAM } from "../common/constants";
import { createDeferredReadableStream } from "./response";
installGlobals();
var redirectStatusCodes = /* @__PURE__ */ new Set([
  301,
  302,
  303,
  307,
  308
]);
function isRedirectResponse(status) {
  return redirectStatusCodes.has(status);
}
function isResponse(value) {
  return value != null && typeof value.status === "number" && typeof value.statusText === "string" && typeof value.headers === "object" && typeof value.body !== "undefined";
}
function convertAlolaRedirectResponse(headers, basename) {
  var newHeaders = new Headers(headers);
  var redirectUrl = headers.get("Location");
  if (basename !== "/") {
    redirectUrl = redirectUrl.replace(basename, "");
  }
  newHeaders.set("X-Alolajs-Redirect", redirectUrl);
  newHeaders.delete("Location");
  return new NodeResponse(null, {
    status: 204,
    headers: newHeaders
  });
}
var createLoaderHeaders = function(requestHeaders) {
  var headers = new Headers();
  var _iteratorNormalCompletion = true, _didIteratorError = false, _iteratorError = void 0;
  try {
    for (var _iterator = Object.entries(requestHeaders)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _step_value = _sliced_to_array(_step.value, 2), key = _step_value[0], values = _step_value[1];
      if (values) {
        if (Array.isArray(values)) {
          var _iteratorNormalCompletion1 = true, _didIteratorError1 = false, _iteratorError1 = void 0;
          try {
            for (var _iterator1 = values[Symbol.iterator](), _step1; !(_iteratorNormalCompletion1 = (_step1 = _iterator1.next()).done); _iteratorNormalCompletion1 = true) {
              var value = _step1.value;
              headers.append(key, value);
            }
          } catch (err) {
            _didIteratorError1 = true;
            _iteratorError1 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion1 && _iterator1.return != null) {
                _iterator1.return();
              }
            } finally {
              if (_didIteratorError1) {
                throw _iteratorError1;
              }
            }
          }
        } else {
          headers.set(key, values);
        }
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return != null) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
  return headers;
};
var createRequest = function(context) {
  var origin = "".concat(context.protocol, "://").concat(context.host);
  var url = new URL(context.url, origin);
  var controller = new AbortController();
  var init = {
    method: context.method,
    headers: createLoaderHeaders(context.headers),
    signal: controller.signal
  };
  if (![
    "GET",
    "HEAD"
  ].includes(context.method.toUpperCase())) {
    init.body = context.req;
  }
  var request = new Request(url.href, init);
  return request;
};
var sendLoaderResponse = function() {
  var _ref = _async_to_generator(function(res, nodeResponse) {
    var _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, _step_value, key, value;
    return _ts_generator(this, function(_state) {
      switch (_state.label) {
        case 0:
          res.statusMessage = nodeResponse.statusText;
          res.statusCode = nodeResponse.status;
          _iteratorNormalCompletion = true, _didIteratorError = false, _iteratorError = void 0;
          try {
            for (_iterator = nodeResponse.headers.entries()[Symbol.iterator](); !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
              _step_value = _sliced_to_array(_step.value, 2), key = _step_value[0], value = _step_value[1];
              res.setHeader(key, value);
            }
          } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion && _iterator.return != null) {
                _iterator.return();
              }
            } finally {
              if (_didIteratorError) {
                throw _iteratorError;
              }
            }
          }
          if (!nodeResponse.body)
            return [
              3,
              2
            ];
          return [
            4,
            writeReadableStreamToWritable(nodeResponse.body, res)
          ];
        case 1:
          _state.sent();
          return [
            3,
            3
          ];
        case 2:
          res.end();
          _state.label = 3;
        case 3:
          return [
            2
          ];
      }
    });
  });
  return function sendLoaderResponse2(res, nodeResponse) {
    return _ref.apply(this, arguments);
  };
}();
var handleRequest = function() {
  var _ref = _async_to_generator(function(param) {
    var context, serverRoutes, routesConfig, query, routeId, entry, basename, end, res, logger, reporter, routes, queryRoute, request, requestContext, response, deferredData, body, init, headers, error, message, cost;
    return _ts_generator(this, function(_state) {
      switch (_state.label) {
        case 0:
          context = param.context, serverRoutes = param.serverRoutes, routesConfig = param.routes;
          query = context.query;
          routeId = query[LOADER_ID_PARAM];
          entry = matchEntry(context.path, serverRoutes);
          if (!routeId || !entry) {
            return [
              2
            ];
          }
          basename = entry.urlPath;
          end = time();
          res = context.res, logger = context.logger, reporter = context.reporter;
          routes = transformNestedRoutes(routesConfig, reporter);
          queryRoute = createStaticHandler(routes, {
            basename
          }).queryRoute;
          request = createRequest(context);
          requestContext = createRequestContext();
          requestContext.set(reporterCtx, reporter);
          _state.label = 1;
        case 1:
          _state.trys.push([
            1,
            3,
            ,
            4
          ]);
          return [
            4,
            queryRoute(request, {
              routeId,
              requestContext
            })
          ];
        case 2:
          response = _state.sent();
          if (isResponse(response) && isRedirectResponse(response.status)) {
            response = convertAlolaRedirectResponse(response.headers, basename);
          } else if (isPlainObject(response) && DEFERRED_SYMBOL in response) {
            deferredData = response[DEFERRED_SYMBOL];
            body = createDeferredReadableStream(deferredData, request.signal);
            init = deferredData.init || {};
            if (init.status && isRedirectResponse(init.status)) {
              if (!init.headers) {
                throw new Error("redirect response includes no headers");
              }
              response = convertAlolaRedirectResponse(new Headers(init.headers), basename);
            } else {
              headers = new Headers(init.headers);
              headers.set("Content-Type", "".concat(CONTENT_TYPE_DEFERRED, "; charset=UTF-8"));
              init.headers = headers;
              response = new NodeResponse(body, init);
            }
          } else {
            response = isResponse(response) ? response : new NodeResponse(JSON.stringify(response), {
              headers: {
                "Content-Type": "application/json; charset=utf-8"
              }
            });
          }
          return [
            3,
            4
          ];
        case 3:
          error = _state.sent();
          message = isRouteErrorResponse(error) ? error.data : String(error);
          if (_instanceof(error, Error)) {
            logger === null || logger === void 0 ? void 0 : logger.error("Error: ".concat(error.name, " ").concat(error.message, " ").concat(error.stack));
          } else {
            logger === null || logger === void 0 ? void 0 : logger.error(message);
          }
          response = new NodeResponse(message, {
            status: 500,
            headers: {
              "Content-Type": "text/plain"
            }
          });
          return [
            3,
            4
          ];
        case 4:
          cost = end();
          reporter.reportTiming("".concat(LOADER_REPORTER_NAME, "-navigation"), cost);
          return [
            4,
            sendLoaderResponse(res, response)
          ];
        case 5:
          _state.sent();
          return [
            2
          ];
      }
    });
  });
  return function handleRequest2(_) {
    return _ref.apply(this, arguments);
  };
}();
export {
  handleRequest,
  isRedirectResponse,
  isResponse
};
